# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django.http import HttpResponse
from django import forms
from django.utils import simplejson
from project.view import FuncView, RootView
from project.fal.migration import Migration
from datetime import datetime
import os
import re


class MigrationForm(forms.Form):

    def __init__(self, locale="ja", *args, **kwargs):
        super(MigrationForm, self).__init__(*args, **kwargs)
        from fields import SrcField, SrcPasswdField, ScheduleField, DateField, TimeField, ForceFullField
        self.fields["src"] = SrcField(
                label=_(u"Migration: Source"),
                required=True)
        self.fields["passwd"] = SrcPasswdField(
                label=_(u"Migration: Source Password"),
                required=True)
        self.fields["sche"] = ScheduleField(
                label=_(u"Migration: Schedule Migration"),
                initial=True,
                required=True)
        self.fields["date"] = DateField(
                label=_(u"Migration: Date"),
                input_formats=[{
                    "ja": "%Y/%m/%d",
                }.get(locale, "%Y/%m/%d")],
                required=False)
        self.fields["time"] = TimeField(
                label=_(u"Migration: Time"),
                required=False)
        self.fields["full"] = ForceFullField(
                label=_(u"Migration: Force Full Migration"),
                required=False)


class MigrationView(FuncView):

    views = ['save', 'save_and_execute', 'switch']

    def view(self, request):
        locale = self.__get_locale()
        initial = self.__get_initial()
        form = MigrationForm(locale=locale, initial=initial)

        template = os.path.join(self.TEMPLATES, "template.html")
        dictionary = {
            "form": form,
            "locale": locale,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_save(self, request):
        locale = self.__get_locale()
        data = request.POST
        form = MigrationForm(locale=locale, data=data)

        with self.processing(constname='ERR_MIGRATION_STATE_CHANGE') as p:
            p.break_if_state_change()
            if form.is_valid():
                migration = Migration()
                result = migration.apply(config={
                    'full': form.cleaned_data['full'],
                    'source': {
                        'name': form.cleaned_data['src'],
                        'passwd': form.cleaned_data['passwd'],
                    },
                    'schedule': {
                        'enable': form.cleaned_data['sche'],
                        'datetime': datetime.combine(
                                form.cleaned_data['date'],
                                form.cleaned_data['time']),
                    },
                })
                p.result(result)
                self.set_message(migration.get_message())

        template = os.path.join(self.TEMPLATES, "template.html")
        dictionary = {
            "form": form,
            "locale": locale,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_save_and_execute(self, request):
        locale = self.__get_locale()
        data = request.POST
        form = MigrationForm(locale=locale, data=data)

        with self.processing(constname='ERR_MIGRATION_STATE_CHANGE') as p:
            p.break_if_state_change()
            if form.is_valid():
                migration = Migration()
                result = migration.apply(config={
                    'full': form.cleaned_data['full'],
                    'source': {
                        'name': form.cleaned_data['src'],
                        'passwd': form.cleaned_data['passwd'],
                    },
                    'schedule': {
                        'enable': form.cleaned_data['sche'],
                        'datetime': datetime.combine(
                                form.cleaned_data['date'],
                                form.cleaned_data['time']),
                    },
                })
                result = result and migration.execute()
                p.result(result)
                self.set_message(migration.get_message())

        template = os.path.join(self.TEMPLATES, "template.html")
        dictionary = {
            "form": form,
            "locale": locale,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_switch(self, request):
        locale = self.__get_locale()
        data = request.POST
        form = MigrationForm(locale=locale, data=data)

        with self.processing(constname='ERR_MIGRATION_STATE_CHANGE') as p:
            p.break_if_state_change()
            if form.is_valid():
                migration = Migration()
                result = migration.switch()
                p.result(result)
                self.set_message(migration.get_message())

        template = os.path.join(self.TEMPLATES, "template.html")
        dictionary = {
            "form": form,
            "locale": locale,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def __get_initial(self):
        initial = {}
        try:
            config = Migration().load()
            initial['full'] = config['full'] or False
            initial['src'] = config['source']['name'] or ''
            initial['passwd'] = config['source']['passwd'] or ''
            initial['sche'] = config['schedule']['enable'] or False
            if isinstance(config['schedule']['datetime'], datetime):
                initial['date'] = config['schedule']['datetime'].date()
                initial['time'] = config['schedule']['datetime'].time()
            else:
                initial['date'] = datetime.today().date()
                initial['time'] = '23:55'
        except:
            pass
        return initial

    def __get_locale(self):
        return "ja"


class MigrationStatusView(RootView):

    def view(self, request):
        statusinfo = Migration().query()
        return {
            self.KEY_RESPONSE: HttpResponse(
                    simplejson.dumps({
                        'status': self.__status(statusinfo['status']),
                        'starttime': self.__fdatetime(statusinfo['starttime']),
                        'endtime': self.__fdatetime(statusinfo['endtime']),
                        'hostname': statusinfo['hostname'],
                        'product': statusinfo['product'],
                        #'elapsedtime': self.__ftime(statusinfo['elapsedtime']),
                        'elapsedtime': statusinfo['elapsedtime'],
                        'progress': statusinfo['progress'],
                    }),
                    mimetype="application/json;charset=UTF-8;"),
        }

    def __status(self, status):
        try:
            return {
                'NO_EXEC': _(u"Migration: Status: Not Execute"),
                'ABORT': _(u"Migration: Status: Abort"),
                'COMPLETE': _(u"Migration: Status: Complete"),
                'COMPLETE:SUCCESS': _(u"Migration: Status: Complete(Success)"),
                'COMPLETE:ERROR': _(u"Migration: Status: Complete(failed)"),
                'PROCESSING': _(u"Migration: Status: Processing"),
            }.get(status, None)
        except:
            return None

    def __fdatetime(self, _datetime):
        try:
            return _datetime.strftime({
                "ja": "%Y/%m/%d %H:%M",
            }.get(self.__get_locale(), "%Y/%m/%d %H:%M"))
        except:
            return None

    #def __ftime(self, _time):
    #    try:
    #        return _time.strftime("%H:%M:%S")
    #    except:
    #        return None

    def __get_locale(self):
        return "ja"


class HostListView(RootView):

    def view(self, request):
        hostlist = []
        try:
            hostlist = Migration().list()

            if len(hostlist) <= 0:
                hostlist = [{'value': '', 'label': _(u"Migration: Host not found.")}]
        except:
            pass
        return {
            self.KEY_RESPONSE: HttpResponse(
                    simplejson.dumps(hostlist),
                    mimetype="application/json;charset=UTF-8;"),
        }


class SwitchableView(RootView):

    def view(self, request):
        switchable = True
        try:
            switchable = Migration().check()
        except:
            pass
        return {
            self.KEY_RESPONSE: HttpResponse(
                    simplejson.dumps({'switchable': switchable}),
                    mimetype="application/json;charset=UTF-8;"),
        }


class SuccessView(RootView):

    def view(self, request):
        success = True
        try:
            #success = Migration().query()['status'] == 'COMPLETE:SUCCESS'
            success = Migration().query()['status'] not in ['NO_EXEC', 'PROCESSING']
        except:
            pass
        return {
            self.KEY_RESPONSE: HttpResponse(
                    simplejson.dumps({'success': success}),
                    mimetype="application/json;charset=UTF-8;"),
        }
